(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*         Jerome Vouillon, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* $Id: translobj.ml,v 1.3 2001/01/05 08:31:08 lefessan Exp $ *)

open Typedtree
open Misc
open Asttypes
open Longident
open Lambda

(* Get oo primitives identifiers *)

let oo_prim name =
  try
    transl_path
      (fst (Env.lookup_value (Ldot (Lident "Oo", name)) Env.empty))
  with Not_found ->
    fatal_error ("Primitive Oo." ^ name ^ " not found.")

let join_prim name =
  try
    transl_path
      (fst (Env.lookup_value (Ldot (Lident "Join", name)) Env.empty))
  with Not_found ->
    fatal_error ("Primitive Join." ^ name ^ " not found.")

(* Collect labels *)

let used_methods = ref ([] : (string * Ident.t) list);;

let meth lab =
  try
    List.assoc lab !used_methods
  with Not_found ->
    let id = Ident.create lab in
    used_methods := (lab, id)::!used_methods;
    id

let reset_labels () =
  used_methods := []

(* Insert labels *)

let string s = Lconst (Const_base (Const_string s))
  
let cst_string s = Const_base (Const_string s)
let cst_int s = Const_base (Const_int s)
  
let cst_cons v l = Const_block (0,[v;l])
let cst_nil = cst_int 0
let cst_pair = cst_cons
let cst_triple v1 v2 v3 = Const_block (0,[v1;v2;v3])
  
let transl_label_init expr = 
  let expr = 
    if (!used_methods = []) then
      expr
    else
    let init = Ident.create "new_method" in
    let expr' =
      Llet(StrictOpt, init, oo_prim "new_method",
        List.fold_right
          (fun (lab, id) expr ->
            Llet(StrictOpt, id, Lapply(Lvar init, [string lab]), expr))
        !used_methods
          expr)
    in
    expr'
  in
  let expr = 
    if !Typejoin.automata_classes = [] then expr else
    let create_class = Ident.create "create_automata_class" in
    Llet(StrictOpt, create_class, join_prim "create_automata_class",
      List.fold_right
        (fun (ident, meths) expr ->
          Printf.printf "--%s--" (Ident.name ident); print_newline ();
          let const = 
            List.fold_right (fun (meth, channel) v ->
                cst_cons (cst_triple (cst_string meth) 
                  (cst_int channel.chan_index)
                  (cst_int (match channel.chan_sync with
                        None -> 0 | _ -> 1)
                        )
                  ) v
            ) meths cst_nil
          in
          let meths = Ident.create "meths" in
          Llet(StrictOpt, meths, Lconst const ,
            Llet(StrictOpt, ident, Lapply(Lvar create_class, [
                  Lvar meths;Lthismodule]), expr)))
      !Typejoin.automata_classes
        expr)    
  in
  reset_labels ();
  expr
  
  