(***********************************************************************)
(*                                                                     *)
(*                     the JoCaml language                             *)
(*                                                                     *)
(*   Fabrice LE FESSANT, projet PARA, INRIA Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(*

  If open_tex fails, due to the absence of a file in the client system,
a location is sent to the server to find the file, and send back a
in_channel for that file. To send the in_channel on the network, a
Jutils.remote_from_in_channel call is used. 

*)

let header = "$Id"

exception Error of string
exception Except

let server_location = Join.here

type rep = 
    Reply of (string * in_channel)
  | Exception of exn

let open_tex filename =
  try
    Files.open_tex filename
  with
    Files.Except -> raise Except
  | Files.Error m ->
      let def waitfile () | file! inc = reply inc in
      let loc getfile 
         do { Join.go server_location; 
              prerr_endline filename;
              let rep =
                try
                  Reply (
                  let (name,inc) = Files.open_tex filename in
                  name,(remote_from_in_channel inc)
                  )
                with
                  e -> Exception e
              in
              file rep
            }
      in
      match waitfile () with
        Reply inc -> inc
      | Exception e -> raise e
      
