(***********************************************************************)
(*                                                                     *)
(*                           xlib for Ocaml                            *)
(*                                                                     *)
(*       Fabrice Le Fessant, projet Para/SOR, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Text
open Keymap
open Efuns
open Simple
open Compile
open Complex
open Window

let mkfile_vars= Str2.regexp "\(\$([a-zA-Z0-9_]*)\)\|\([a-zA-Z0-9_]+=\)"
let mkfile_target= Str2.regexp "^.*:"
let mkfile_rules= Str2.regexp "^\t.*$"

let makefile_color buf =
  let location = buf.buf_location in
  let red_attr = make_attr (get_color location "red") 1 0 false in
  let yellow_attr = make_attr (get_color location "yellow") 1 0 false in
  let blue_attr = make_attr (get_color location "blue") 1 0 false in
  let gray_attr = make_attr (get_color location "gray") 1 0 false in
  color buf mkfile_rules false red_attr;
  color buf mkfile_target false yellow_attr;
  color buf mkfile_vars false blue_attr;()
 
let c_c = (ControlMap,Char.code 'c')
let install buf =
  Buffer.set_major_mode buf "Makefile";
  let map = buf.buf_local_map in
  Keymap.add_prefix map [c_c];  
  interactive (Buffer buf) [c_c; ControlMap, Char.code 'c'] 
    "compile" (compile c_find_error);
  interactive (Buffer buf) [ControlMap, Char.code 'l']
    "makefile-colors" (fun frame -> makefile_color frame.frm_buffer);
  makefile_color buf;
  Keymap.add_binding map [NormalMap, XK.xk_Tab] (fun frame -> 
	insert_string frame "\t"; ()
      )
 
let _ =  
  Efuns.add_start_hook (fun location ->
      let alist = get_local_var (Location location) Buffer.modes_alist in
      add_interactive (Location location) "makefile-mode" 
      (fun frame -> install frame.frm_buffer);
    set_local_var (Location location) Buffer.modes_alist 
      (("[Mm]akefile[<]?[0-9]*[>]?",install)
       :: alist)
      )   
