open Unix
open Graphics
open ThreadUnix
open Thread
open Debug
(*open Printf*)

let _ = Debug.log (open_out "/tmp/Joplug-server-log")

let log s = 
  let date = localtime (time ()) in
  Debug.print2 's' "%s --> %s\n"
    (Printf.sprintf "%d/%d/%d %d:%d:%d " 
       date.tm_mday date.tm_mon (date.tm_year+1900) 
       date.tm_hour date.tm_min date.tm_sec) s

let _ = log "start"

type command =
    Window of int * int * int * int
  | Kill

type newinst = True | False

let read_token fd =
  let rec read_rec buf bufsize offs =
    let n = read fd buf offs 1 in
      if n = 0 then raise End_of_file
      else if List.mem buf.[offs] ['\n'; ' '; '\t']
           then String.sub buf 0 offs
	   else let offs = succ offs in
	      if offs = bufsize 
      	      then read_rec (buf ^ String.create 32) (bufsize + 32) offs
	      else read_rec buf bufsize offs in
  read_rec (String.create 32) 32 0 

let read_int fd =
  int_of_string (read_token fd)

let read_string fd =
  let len = read_int fd in
  let s = String.create len in
    read fd s 0 len;
    s

let read_ack fd =
  read fd " " 0 1;
  ()

let write_int fd n =
  let s = string_of_int n ^ " " in
    write fd s 0 (String.length s);
    ()

let write_string fd s =
  write_int fd (String.length s);
  write fd s 0 (String.length s);
  ()

let write_ack fd =
  write fd " " 0 1;
  ()
;;

let wait_key() =
try
while true do 
if key_pressed() then if read_key() = 'a' then raise End_of_file done
with
| End_of_file -> ()
;; 

let nwino = ref 0;;
let newinst = ref false;;

let instance = ref 0
let instances = Hashtbl.create 13



let def new_instance (server,port,name) | newclient (client,host,(args:string list)) = 
  let instance = incr instance; !instance in
  let father_log = log in
  let def log s = father_log s;
    reply ()
  in
  let loc location def
     command(cmd) | newinst! (newp) =
      match (cmd,newp) with
        (Window(wx,wy,ww,wh),True) -> 
          {
          log "client: first Window";
          let param = " "^(string_of_int ww)^"x"^(string_of_int wh)^
            "+"^(string_of_int wx)^"+"^(string_of_int wy) 
          in
          log ("opengraph with "^param);
          begin
            try
              Debug.print0 's' "opengraph...";
              open_graph param
            with
              e -> 
                Debug.print0 's' "Exception in opengraph";
                log (Printexc.to_string e;);
                raise e
          end;
          Debug.print0 's' "opengraph OK";
          log "Graph open";
          {
          reply (let win = get_win () in log ("Get win "^(string_of_int win)); win) to command | 
          {
          log "lookup...";
          let (play: Join.location * string * string list -> unit ) = 
            Ns.lookup name vartype in
          log "...lookup OK";
          play (location,host,[]);
          log "client started";
          newinst(False)
          } 
          } 
          }   
      | (Window _, False) -> { log "Client: second window"; reply (get_win ()) | newinst(False) }
      | Kill,_  -> 
           { log "Client: kill"; close_graph ();
          (* reply and exit *)
           { reply 0 | newinst newp | {delay 2.0; Pervasives.exit 0;} }}
   do {
     Join.go client;
     log "Client: migrated";
     set_plugin true;
     newinst(True)}
   in
  Hashtbl.add instances instance command;
  reply instance to new_instance | reply () to newclient
;;

let ((_,local_port),_) = Space.local_id
let _ = Join.setLocalService "newclient" newclient vartype



let _ =

  let pid = ref 0 in
  Arg.parse [
  "-pid", Arg.Int (fun i -> pid := i), "Netscape pid"
]
    (fun s -> ())
    "Usage: Jopluge <options>";

 (* synchro : we create the socket, and then fork() so that the parent
    returns, and the child does the work. 
    Our caller can invoke system("calves") and be sure that when the
    call returns, the socket accepts connections
    *)

  let s = socket PF_UNIX SOCK_STREAM 0 in
  setsockopt s SO_REUSEADDR true;
     (* we might have several NN running *)
  let sockname = 
    Filename.concat (Filename.concat (Sys.getenv "HOME") ".netscape")
      (Printf.sprintf "Joplug-%08d" !pid) in

     (* This creates the socket *)
  bind s (ADDR_UNIX sockname);
  listen s 10;

  if fork() <> 0 then
    begin
      log "close Parent";
      Pervasives.exit 0
    end;
  
  let fd, sock_addr = accept s in
  
  try
    while true do
      match read_string fd with
        "INIT" -> (* the master connection *)
          log "INIT";
          
	  while true do
            match read_string fd with
	      "NEW" ->
                log "NEW";
      (* NEW <s>(mimtype) <i>(argc) <s><s>(argv[i]) ... *)
      (* Responds: <i>(instid) *)
 	        let argc = read_int fd in
	        let args = ref [] in
	        for i = 0 to argc - 1 do
	          let name = read_string fd in
	          let value = read_string fd in
	          args := (name, value) :: !args
	        done;
	        let argm = List.map (function (name,value) -> 
                  String.lowercase name, value) !args in
                let cmd = Printf.sprintf "jogclient -direct -ns_port %d &" local_port in
                log cmd;
                system cmd;
                log "OK";
                begin
                  try
	            let port = int_of_string (List.assoc "jnsport" argm) in
	            let mname = (List.assoc "jnsname" argm) in
                    let instid = new_instance 
                        (mname,port,(List.assoc "remotep" argm)) in
                    let _ = log (Printf.sprintf "NEW reply : %d" instid) in
                    write_int fd instid;
                  with
                    Not_found -> 
                      log "Error in NEW parsing";
                      write_int fd (-1)
                end
            | "WIN" ->
                log "WIN";
      (* WIN <i>(instid) <i>(x) <i>(y) <i>(w) <i>h *)
      (* Responds: <s>(windowid) *)
                let instid = read_int fd in
                let _ = log (Printf.sprintf "WIN instid : %d" instid) in
	        let wx=read_int fd in
	        let wy=read_int fd in
	        let ww=read_int fd in
	        let wh=read_int fd in
                begin
                  try
                    let instance = Hashtbl.find instances instid in
                    let win = instance (Window(wx,wy,ww,wh))
                    in
                    log (Printf.sprintf "WIN reply : %d" win);
                    write_int fd win
                  with 
                    Not_found -> log "Instance not found"
                end
            | "KILL" ->
                log "KILL";
      (* KILL <i>(instid) *)
      (* Responds: <A>    *)
                let instid = read_int fd in
                let _ = log (Printf.sprintf "WIN instid : %d" instid) in
                begin
                  try
	            let instance = Hashtbl.find instances instid in
                    spawn {instance Kill;}
                   (* instance might have been already killed *)
                  with _ -> ()
                end;
                Hashtbl.remove instances instid;
 	        write_ack fd;
                log "KILL ACK"
            | "NOWIN" ->
      (* NOWIN <i>(instid) *)
      (* Responds: <A> *)
                log "NOWIN";
                let instid = read_int fd in
                write_ack fd;
                log "KILL ACK"
            | "PRINT" ->
      (* PRINT <i>(instid) *)
      (* Responds: <A> *)
                log "PRINT";
                let instid = read_int fd in
                write_ack fd;
                log "PRINT ACK"
            | s -> log ("Unxepected query INIT "^s)
          done
      | s -> log ("Unexpected query "^s)
    done
  with
    End_of_file ->
      Hashtbl.iter (fun _ instance -> let _ = instance Kill in ()) instances;
      close fd;
      Pervasives.exit 0 (* so that at_exit hooks are called *)

