open Jtypes

let _ = CdDebug.cadmiumFlag := true
let char = 'z'


(* Operation on objects *) (* original was in oo.ml *)

let copy (o: < .. >) = 
  let o' = Obj.repr o in
  let (tag, locator, space, location, local, gc_stub) = 
    Jcompat.get_stub_info o' in
  if not local then
    (* failwith "Can not copy non local object"; *)
    
  let sz = Obj.size o' in
  let r = 
    let self_loc = Jprims.self_location () in
    Jprims.create_new_object Space.local_space 
      self_loc sz (Jprims.get_location_module o)
  in
  for i = 5 to sz - 1 do Obj.set_field r i (Obj.field o' i) done;
  Obj.obj r


(* Parallel list treatment *)
(* let rec parallel_iter f (* asynchronous channel*) = function *)
(*     [] -> () *)
(*   | a::l -> spawn{f a}; parallel_iter f l;; *)

let rec parallel_iter f = function
    [] -> ()
  | a::l -> f a; parallel_iter f l;;

(* Cache lookup table operation *)

let print_lookup_uids () = (* prints all uids of the cache lookup table *)
  let Some (Contents tbl) = Space.local_space.cdCache (* *** pattern-matching is not exhaustive *)
  in (let give_list uid (stub,object) = CdUtils.print_uid uid
      in
 	CdDebug.print "cdCache.print_uids: cache got following uids\n";
	Hashtbl.iter give_list tbl)
 
let print_lookup_object_get_name () = (* call #get_name for all local objects of the cache - testing purpose *)
  let Some (Contents tbl) = Space.local_space.cdCache (* *** pattern-matching is not exhaustive *)
  in (let do_print uid (stub,object) = print_string ((Obj.magic object)#get_name); print_newline ()
      in
	CdDebug.print "cdCache.print_lookup_object_get_name: cache got following objects\n";
	Hashtbl.iter do_print tbl)
       
let add stub =
  let uid = CdUtils.get_uid stub
  in 
  let Some (Contents tbl) = Space.local_space.cdCache (* *** pattern-matching is not exhaustive *)
  in
    try
      (* lock tbl *) (* Jprims.set_critical_section "Cache tbl lock" true; *) (* enough??? *)
      Hashtbl.find tbl uid;
      Debug.print0 char "Uid found";
      (* if found => unlock tbl *)
    with
	Not_found -> 
	  Hashtbl.add tbl uid (stub,stub); (* create the entry uid, fill the other fields with anything *)
	  (* unlock tbl *)
	  (* launch copy and migration of the target object in the cache *)
	  let object = (Obj.magic stub)#copy_yourself
	  in
(* 	    (Obj.magic stub)#get_name (); *)
(* 	    (Obj.magic stub)#set_name "Meeting original"; *)
(* 	    (Obj.magic stub)#get_name (); *)
(* 	    (Obj.magic object)#get_name (); *)
(* 	    (Obj.magic object)#set_name "Meeting copy"; *)
(* 	    (Obj.magic object)#get_name (); *)
(* 	    (Obj.magic stub)#get_name (); *)
	    (Obj.magic object)#coucou "--------------------------------Try to talk with the copy";
	    (Obj.magic object)#migrate_to Space.local_location; (* should be the cache *)
	    Debug.print0 char "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Migrate_to called\n";
	    let local_object = ((Obj.magic object)#check_ref)
	    in
	      Debug.print0 char "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! check_ref called\n";
              (* lock tbl *)
	      Hashtbl.remove tbl uid; (* removed stupid stuff used to lock the entry *)
	      Hashtbl.add tbl uid (stub,local_object) (* add the correct references *)
    	      (* unlock tbl *)


(* let def add (stub) =  *)
(*   { *)
(*     Debug.print0 char "Uid found"; *)
(*     let uid = CdUtils.get_uid stub *)
(*     in  *)
(*     let Some (Contents tbl) = Space.local_space.cdCache (* *** pattern-matching is not exhaustive *) *)
(*     in *)
(*       ( *)
(*       try *)
(*         (* lock tbl *) (* Jprims.set_critical_section "Cache tbl lock" true; *) (* enough??? *) *)
(* 	Hashtbl.find tbl uid; *)
(* 	Debug.print0 char "Uid found"; *)
(* 	(* if found => unlock tbl *) *)
(*       with *)
(* 	  Not_found ->  *)
(* 	    Hashtbl.add tbl uid (stub,stub); (* create the entry uid, fill the other fields with anything *) *)
(* 	    (* unlock tbl *) *)
(* 	    (* launch copy and migration of the target object in the cache *) *)
(* 	    let object = (Obj.magic stub)#copy_yourself *)
(* 	    in *)
(* 	      (Obj.magic object)#migrate_to Space.local_location; (* should be the cache *) *)
(* 	      let local_object = ((Obj.magic object)#check_ref) *)
(* 	      in *)
(* 	        (* lock tbl *) *)
(* 	      	Hashtbl.remove tbl uid; (* removed stupid stuff used to lock the entry *) *)
(* 		Hashtbl.add tbl uid (stub,local_object); (* add the correct references *) *)
(*     	        (* unlock tbl *) *)
(*       );   *)
(*   } *)
  
let addObjects stubList = (* fills the cache lookup table and copies the objects locally *)
                          (* extract object references (i.e. stubs) from stubList *)  
  parallel_iter add stubList;
  print_lookup_uids ();
  print_lookup_object_get_name ();  
























(* Orig *)
(* let addObjects stubList = (* fills the cache lookup table and copies the objects locally *) *)
(*                           (* extract object references (i.e. stubs) from stubList *)   *)
(*   Debug.print0 char "cdCache.addObjects\n"; *)
(*   Debug.print1 char "cdCache.addObjects: stubList length: %d\n" (List.length stubList); *)
(*   let add stub =  *)
(*     let uid = CdUtils.get_uid stub *)
(*     in  *)
(*       Debug.print0 char "cdCache.addObjects: treat uid: \n"; CdUtils.print_uid uid; *)
(*       Debug.print0 char "cdCache: just after\n"; *)
(*       let Some (Contents tbl) = Space.local_space.cdCache (* *** pattern-matching is not exhaustive *) *)
(*       in  *)
(* 	try  *)
(* 	  Debug.print0 char "cdCache: let...in...try...\n"; *)
(* 	  Hashtbl.find tbl uid; *)
(* 	  Debug.print0 char "Executed if found?\n" *)
(* 	with *)
(* 	    Not_found ->  *)
(* 	      Debug.print0 char "Not found\n"; *)
(* 	      Debug.print0 char "It is here! *****************************************************\n"; *)
(* 	      (Obj.magic stub)#coucou "It is me!"; *)
(* 	      Debug.print0 char "It ends! ********************************************************\n"; *)
(* 	      let object = Obj.repr (copy (Obj.magic stub)) (* temporary stuff *) *)
(*                             (* copy the object *)(* to be executed in parallel *) *)
(* 	      in  *)
(* 		Debug.print0 char "let...in in not found\n"; *)
(* 		Hashtbl.add tbl uid (stub,object); (* *** correct??? *) *)
(* 		Debug.print0 char "do copy just above\n" *)
(*   in  *)
(*     List.iter add stubList; *)
(*     Debug.print0 char "cdCache.addObjects: all cache uids\n"; *)
(*     print_lookup_uids (); *)
(*     Debug.print0 char "cdCache.addObjects end\n" *)
  
    
