open Jtypes

external remove_thread: local_object -> unit = "remove_thread"

let rec halt_stub stub =
  match stub.local with
    RemoteObject 
  | FailedObject _ -> ()
  | LocalThread thread -> 
      remove_thread stub.local;
      stub.local <- FailedObject Thread
  | LocalAutomata automata -> 
      stub.local <- FailedObject Automata
  | LocalLocation location -> 
      let object_type =  
        if Obj.tag (Obj.repr stub) = Tag.location then Location
        else
          let class_info = Mop.get_class_info stub in
          Object (Space.local_id,
                  class_info.Mop.class_type,
                  class_info.Mop.class_sig,
	          class_info.Mop.table.Mop.size)
      in
      let threads = Jprims.extract_location_threads stub in
      let defs = Weak.set_array location.def_weakset in
      let locs = Weak.set_array location.loc_weakset in
      Array.iter (fun stub -> halt_stub stub) threads;
      Array.iter (fun stub -> halt_stub stub) defs;
      Array.iter (fun stub -> halt_stub stub) locs;
      Jprims.remove_from_father stub;
      stub.local <- FailedObject object_type
      

let halt_location location = halt_stub location

let halt_space space state =
  space.space_state <- state;
  begin
    match space.link_in with
      NoHandler -> ()
    | Handler handler ->
        close_in handler.channel;
        Thread.kill handler.thread
  end;
  space.link_in <- NoHandler;
  begin
    match space.link_out with
      NoConnection -> ()
    | Connecting thread -> Thread.kill thread
    | Connected connection ->
        close_out connection.channel;
        Thread.kill connection.thread;
  end;
  space.link_out <- NoConnection;
  Jprims.cache_clean space.cache_out;
  Fifo.clear space.queue;
  Jprims.cache_clean space.cache_in;
  Jprims.cache_clean space.cache_out;
  (* No more DGC support: remote objects may get collected *)
  Hashtbl.clear space.scions_hash;
  space.scions_table <- [||];
  space.free_list <- [||];
  space.free_slots <- 0;
  space.stubs_table <- Weak.create 0;
  ()

let fail obj = 
  let stub = Jprims.extract_stub obj in
  Jprims.wait_on_fail stub
